"""
Example client script for MHCII3D REST service (Python3). 
Our service provides either binding predictions for (i) a single peptide, 
(ii) a set of peptides, or (iii) a scan of protein sequences for binding 
peptides. 

This script includes three examples for accessing our service:
- The first example (starting at line 75) shows a request for scoring 
  a single peptide. 
- In the second example (starting at line 108) the evaluation of multiple 
  peptides is demonstrated. 
- Finally, in the third example (starting at line 140) a protein sequence scan
  is performed. 
"""

import urllib.request
import json
from time import sleep

url = 'https://pbwww.che.sbg.ac.at/biodapps2/%s'

########################################################
#helper functions
########################################################

#request a job id
def requestJobId ():
    jobId = None
    if jobId == None:
        urlJob = url % "jobid"
        data = {"service":"mhcii3d"}
        data = json.dumps(data).encode("utf8")
        req = urllib.request.Request(urlJob, data=data, headers={'Content-Type': 'application/json'})
        response = urllib.request.urlopen(req)
        result = json.load(response)
        jobId = result["jobid"]
    print("jobid: %s" % jobId)
    return jobId

#request the status of a job
#-1 ... waiting (not started)
# 0 ... running 
# 1 ... finished
# 2 ... failed  
def requestStatus(jobid):
    data = {
        "jobid": jobid
        }
    data = json.dumps(data).encode("utf8")
    req = urllib.request.Request(url % "status", data=data, headers={'Content-Type': 'application/json'})
    response = urllib.request.urlopen(req)
    status = json.loads(response.read().decode())
    return status

#regularly check the current job status until it finished.
def checkJob (status, jobid, sleeptime=5):
    while status["status_id"] < 1:
        sleep(sleeptime)
        status= requestStatus(jobid)
        print("status:", status["status_id"], status["status"])
    return status

#request results
def requestJobResult (jobid):
    data = {
        "jobid": jobid
        }
    data = json.dumps(data).encode("utf8")
    req = urllib.request.Request(url % "result", data=data, headers={'Content-Type': 'application/json'})
    response = urllib.request.urlopen(req)
    result = json.loads(response.read().decode())
    return result

########################################################
#1st example: score a single peptide sequence
########################################################

print("\n1st example - score a single peptide sequence\n")

#request a job id
jobId = requestJobId()

#setup data
data = {"service": "mhcii3d", 
        "jobid": jobId, 
        "param": {
            "peptide": "QTKIQYVIRAQLHVG",
            "beta_allele": "DRB1*08:01"
        }
    }

data = json.dumps(data).encode("utf8")
#call service
req = urllib.request.Request(url % "run", data=data, headers={'Content-Type': 'application/json'})
response = urllib.request.urlopen(req)
status = json.loads(response.read().decode())

if status["status_id"] < 1:  #job is still running 
	status = checkJob(status, jobId)

if status["status_id"] == 1: #job finished successfully 
	result = requestJobResult(jobId)
	for r in result:
		print(r["beta_allele"], r["peptide"], r["binding_peptide"], r["score"], r["rank"], r["ic50_pred"])
		

########################################################
#2nd example: score a list of peptide sequences 
########################################################

print("\n\n2nd example - score a list of peptide sequences\n")

jobId = requestJobId()
#setup data
data = {"service": "mhcii3d", 
        "jobid": jobId, 
        "param": {
            "peptide_list": ["QTKIQYVIRAQLHVG", "TNDRKWCFEGPEEHE"], #define multiple peptides 
            "beta_allele": "DRB1*08:01"
        }
    }

data = json.dumps(data).encode("utf8")
#call service
req = urllib.request.Request(url % "run", data=data, headers={'Content-Type': 'application/json'})
response = urllib.request.urlopen(req)
status = json.loads(response.read().decode())

if status["status_id"] < 1:  #job is still running 
	status = checkJob(status, jobId)

if status["status_id"] == 1: #job finished successfully 
	result = requestJobResult(jobId)
	for r in result:
		print(r["beta_allele"], r["peptide"], r["binding_peptide"], r["score"], r["rank"], r["ic50_pred"])



########################################################
#3rd example: scan of a protein sequence
########################################################

print("\n\n3nd example - scan of a protein sequence\n")

jobId = requestJobId()

sequence = "AHCIGITDRDFIEGVHGGTWVSATLEQDKCVTVMAPDKPSLDISLQTVAIDGPAEARKVCYSAVLTHVKINDKCPSTGEAHLAEENDGDNACKRTYSDRGWGNGCGLFGKGSIVACAKFTCAKSMSLFEVDQTKIQYVIRAQLHVGAKQE" #PDB entry 6IW5_1; first 150 residues
betaAllele  = "DRB1*08:01"

data = {"service": "mhcii3d", 
        "jobid": jobId, 
        "param": {
            "protein_sequence": sequence,
            "beta_allele": betaAllele
        }
    }

data = json.dumps(data).encode("utf8")
#call service
req = urllib.request.Request(url % "run", data=data, headers={'Content-Type': 'application/json'})
response = urllib.request.urlopen(req)
status = json.loads(response.read().decode())

if status["status_id"] < 1:  #job is still running 
	status = checkJob(status, jobId, 10)

if status["status_id"] == 1: #job finished successfully 
	result = requestJobResult(jobId)
	for r in result:
		print(r["beta_allele"], r["peptide"], r["binding_peptide"], r["score"], r["rank"], r["ic50_pred"])

